package com.bean;

import javax.faces.context.FacesContext;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.event.AbortProcessingException;

import org.richfaces.component.state.TreeState;
import org.richfaces.event.NodeSelectedEvent;

public class TreeBean {

	private TreeNode rootNode = null;
	private TreeNode[] rootNodes = null;

	private TreeState treeState;

	private static final String DATA_PATH = "/simple-tree-data.properties";

	public TreeNode getRootNode() {
		if (rootNode == null) {
			loadTree();
		}

		return rootNode;
	}

	public void setRootNode(TreeNode rootNode) {
		this.rootNode = rootNode;
	}

	public TreeNode[] getRootNodes() {
		if (rootNodes == null) {
			rootNodes = this.getRootNode().getChildren();
		}
		return rootNodes;
	}

	public void setRootNodes(TreeNode[] rootNodes) {
		this.rootNodes = rootNodes;
	}

	public TreeState getTreeState() {
		return treeState;
	}

	public void setTreeState(TreeState treeState) {
		this.treeState = treeState;
	}

	private void addNodes(String path, TreeNode node, Properties properties) {
		boolean end = false;
		int counter = 1;

		while (!end) {
			String key = path != null ? path + '.' + counter : String
					.valueOf(counter);

			String value = properties.getProperty(key);
			if (value != null) {
				TreeNode newNode = new TreeNode(value);
				newNode.setParent(node);
				node.addNode(newNode);
				addNodes(key, newNode, properties);
				counter++;
			} else {
				end = true;
			}
		}
	}

	private void loadTree() {
		System.out.println("TreeBean -> loadTree()");
		FacesContext facesContext = FacesContext.getCurrentInstance();
		ExternalContext externalContext = facesContext.getExternalContext();
		InputStream dataStream = externalContext.getResourceAsStream(DATA_PATH);
		try {
			Properties properties = new Properties();
			properties.load(dataStream);

			rootNode = new TreeNode("root");
			addNodes(null, rootNode, properties);
			// manual tree extension

			TreeNode dummy = new TreeNode("Sichtlinie");
			int count = 0;
			for (int i = 1; i <= 2; i++) {
				count++;
				TreeNode node = new TreeNode("i" + i);
				int counter = 1;
				nodeProcess(node,counter);
				/*for (int j = 1; j <= 100; j++) {
					count++;
					if (count % 1000 == 0) {
						System.out.println(count);
					}
					TreeNode child = new TreeNode("j" + j);
					child.setParent(node);
					node.addNode(child);
				}*/
				node.setParent(dummy);
				dummy.addNode(node);
			}
			rootNode.addNode(dummy);

		} catch (IOException e) {
			e.printStackTrace();
			throw new FacesException(e.getMessage(), e);

		} finally {
			if (dataStream != null) {
				try {
					dataStream.close();
				} catch (IOException e) {
					e.printStackTrace();
					externalContext.log(e.getMessage(), e);
				}
			}
		}

	}

	public void processSelection(NodeSelectedEvent arg0)
			throws AbortProcessingException {
		System.out.println("SimpleTreeBean -> processSelection fired by "
				+ arg0.getComponent().getId());

	}
	
	public void nodeProcess(TreeNode node, int counter) throws AbortProcessingException {
		
		TreeNode child = new TreeNode("counter::::" + counter);
		child.setParent(node);
		node.addNode(child);
		if(counter < 101){
			counter++;
			nodeProcess(child,counter);
		}

}
	

}
